# Copyright 2015 Huawei Technologies Co., Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse

from openstack_dashboard.test import helpers

from mistraldashboard import api
from mistraldashboard.test import helpers as test

INDEX_URL = reverse('horizon:mistral:workflows:index')
CREATE_URL = reverse('horizon:mistral:workflows:create')
UPDATE_URL = reverse('horizon:mistral:workflows:update')


class WorkflowsTest(test.TestCase):

    @helpers.create_mocks({api: ('workflow_list',)})
    def test_index(self):
        self.mock_workflow_list.return_value =\
            self.mistralclient_workflows.list()
        res = self.client.get(INDEX_URL)

        self.assertTemplateUsed(res, 'mistral/workflows/index.html')
        self.assertItemsEqual(res.context['table'].data,
                              self.mistralclient_workflows.list())
        self.mock_workflow_list.assert_called_once_with(
            helpers.IsHttpRequest())

    def test_create_get(self):
        res = self.client.get(CREATE_URL)
        self.assertTemplateUsed(res, 'mistral/workflows/create.html')

    @helpers.create_mocks({api: ('workflow_validate',
                                 'workflow_create')})
    def test_create_post(self):
        workflow = self.mistralclient_workflows.first()
        self.mock_workflow_validate.return_value = {'valid': True}
        self.mock_workflow_create.return_value = workflow

        url = reverse('horizon:mistral:workflows:select_definition')
        res = self.client.get(url)
        self.assertTemplateUsed(
            res,
            'mistral/workflows/select_definition.html'
        )
        form_data = {
            'definition_source': 'raw',
            'definition_data': workflow.definition
        }
        res = self.client.post(url, form_data)

        self.assertTemplateUsed(res, 'mistral/workflows/create.html')
        self.mock_workflow_validate.assert_called_once_with(
            helpers.IsHttpRequest(),
            workflow.definition
        )

        form_data = {
            'definition': workflow.definition
        }
        res = self.client.post(CREATE_URL, form_data)
        self.assertNoFormErrors(res)
        self.assertEqual(res.status_code, 302)
        self.assertRedirectsNoFollow(res, INDEX_URL)

        self.mock_workflow_create.assert_called_once_with(
            helpers.IsHttpRequest(),
            workflow.definition
        )

    def test_update_get(self):
        res = self.client.get(UPDATE_URL)
        self.assertTemplateUsed(res, 'mistral/workflows/update.html')

    @helpers.create_mocks({api: ('workflow_validate',
                                 'workflow_update')})
    def test_update_post(self):
        workflow = self.mistralclient_workflows.first()
        self.mock_workflow_validate.return_value = {'valid': True}
        self.mock_workflow_update.return_value = workflow

        url = reverse('horizon:mistral:workflows:change_definition')
        res = self.client.get(url)
        self.assertTemplateUsed(
            res,
            'mistral/workflows/select_definition.html'
        )
        form_data = {
            'definition_source': 'raw',
            'definition_data': workflow.definition
        }
        res = self.client.post(url, form_data)

        self.assertTemplateUsed(res, 'mistral/workflows/update.html')
        self.mock_workflow_validate.assert_called_once_with(
            helpers.IsHttpRequest(),
            workflow.definition
        )

        form_data = {
            'definition': workflow.definition
        }
        res = self.client.post(UPDATE_URL, form_data)
        self.assertNoFormErrors(res)
        self.assertEqual(res.status_code, 302)
        self.assertRedirectsNoFollow(res, INDEX_URL)

        self.mock_workflow_update.assert_called_once_with(
            helpers.IsHttpRequest(),
            workflow.definition
        )

    @helpers.create_mocks({api: ('workflow_list',
                                 'workflow_delete')})
    def test_delete_ok(self):
        workflows = self.mistralclient_workflows.list()
        self.mock_workflow_list.return_value = workflows
        self.mock_workflow_delete.return_value = None

        data = {'action': 'workflows__delete',
                'object_ids': [workflows[0].name]}

        res = self.client.post(INDEX_URL, data)

        self.mock_workflow_delete.assert_called_once_with(
            helpers.IsHttpRequest(),
            workflows[0].name
        )
        self.mock_workflow_list.assert_called_once_with(
            helpers.IsHttpRequest())
        self.assertNoFormErrors(res)
        self.assertRedirectsNoFollow(res, INDEX_URL)

    @helpers.create_mocks({api: ('workflow_get',)})
    def test_detail(self):
        workflow = self.mistralclient_workflows.list()[0]
        self.mock_workflow_get.return_value = workflow
        url = reverse('horizon:mistral:workflows:detail',
                      args=[workflow.name])
        res = self.client.get(url)

        self.assertTemplateUsed(res, 'mistral/workflows/detail.html')
        self.mock_workflow_get.assert_called_once_with(
            helpers.IsHttpRequest(), workflow.name)



# Copyright 2014 - StackStorm, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy

from horizon import exceptions
from horizon import tables

from mistraldashboard import api
from mistraldashboard.default import smart_cell
from mistraldashboard.default.utils import humantime
from mistraldashboard.default.utils import label

smart_cell.init()


class DeleteExecution(tables.DeleteAction):
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Execution",
            u"Delete Executions",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Execution",
            u"Deleted Executions",
            count
        )

    def delete(self, request, execution_name):
        api.execution_delete(request, execution_name)


class CancelExecution(tables.BatchAction):
    name = "cancel execution"
    classes = ("btn-danger",)

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Cancel Execution",
            u"Cancel Executions",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Canceled Execution",
            u"Canceled Executions",
            count
        )

    def allowed(self, request, instance):
        if instance.state == "RUNNING":
            return True
        return False

    def action(self, request, obj_id):
        api.execution_update(request, obj_id, "state", "ERROR")


class PauseExecution(tables.BatchAction):
    name = "pause execution"

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Pause Execution",
            u"Pause Executions",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Paused Execution",
            u"Paused Executions",
            count
        )

    def allowed(self, request, instance):
        if instance.state == "RUNNING":
            return True
        return False

    def action(self, request, obj_id):
        api.execution_update(request, obj_id, "state", "PAUSED")


class ResumeExecution(tables.BatchAction):
    name = "resume execution"

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Resume Execution",
            u"Resume Executions",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Resumed Execution",
            u"Resumed Executions",
            count
        )

    def allowed(self, request, instance):
        if instance.state == "PAUSED":
            return True
        return False

    def action(self, request, obj_id):
        api.execution_update(request, obj_id, "state", "RUNNING")


class UpdateDescription(tables.LinkAction):
    name = "updateDescription"
    verbose_name = _("Update Description")
    url = "horizon:mistral:executions:update_description"
    classes = ("ajax-modal",)


class UpdateRow(tables.Row):
    ajax = True

    def get_data(self, request, id):
        try:
            instance = api.execution_get(request, id)
        except Exception as e:
            msg = _("Unable to get execution by ID %(id)s: %(e)s.") % {
                'id': id, 'e': str(e)
            }
            exceptions.handle(request, msg)

        return instance


class ExecutionsTable(tables.DataTable):

    def getHoverHelp(data):
        if hasattr(data, 'state_info') and data.state_info:

                return {'title': data.state_info}

    STATE_STATUS_CHOICES = (
        ("success", True),
        ("error", False),
        ("paused", False),
        ("delayed", None),
        ("running", None),
    )

    STATUS_DISPLAY_CHOICES = (
        ("success", _("Success")),
        ("error", _("Error")),
        ("paused", _("Paused")),
        ("delayed", _("Delayed")),
        ("running", _("Running")),
    )

    id = tables.Column(
        "id",
        verbose_name=_("ID"),
        link="horizon:mistral:executions:detail"
    )

    workflow_name = tables.Column(
        "workflow_name",
        verbose_name=_("Workflow")
    )

    task = tables.Column(
        "task",
        verbose_name=_("Tasks"),
        empty_value=_("View"),
        link="horizon:mistral:tasks:execution"
    )

    input = tables.Column(
        "",
        verbose_name=_("Input"),
        empty_value=_("View"),
        link="horizon:mistral:executions:input",
        link_classes=("ajax-modal",)
    )

    output = tables.Column(
        "",
        verbose_name=_("Output"),
        empty_value=_("View"),
        link="horizon:mistral:executions:output",
        link_classes=("ajax-modal",)
    )

    created_at = tables.Column(
        "created_at",
        verbose_name=_("Created at"),
        filters=[humantime]
    )

    updated_at = tables.Column(
        "updated_at",
        verbose_name=_("Updated at"),
        filters=[humantime]
    )

    state = tables.Column(
        "state",
        verbose_name=_("State"),
        filters=[label],
        status=True,
        status_choices=STATE_STATUS_CHOICES,
        display_choices=STATUS_DISPLAY_CHOICES,
        cell_attributes_getter=getHoverHelp
    )

    class Meta(object):
        name = "executions"
        verbose_name = _("Executions")
        status_columns = ["state"]
        row_class = UpdateRow
        table_actions = (DeleteExecution, tables.FilterAction)
        row_actions = (DeleteExecution, UpdateDescription,
                       PauseExecution, CancelExecution,
                       ResumeExecution, DeleteExecution)



# Copyright 2014 - StackStorm, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.conf.urls import url  # noqa

from mistraldashboard.workbooks import views

WORKBOOKS = r'^(?P<workbook_name>[^/]+)/%s$'

urlpatterns = [
    url(r'^$', views.IndexView.as_view(), name='index'),
    url(r'^select_definition$',
        views.SelectDefinitionView.as_view(),
        name='select_definition'),
    url(r'^change_definition$',
        views.ChangeDefinitionView.as_view(),
        name='change_definition'),
    url(r'^create$', views.CreateView.as_view(), name='create'),
    url(r'^update$', views.UpdateView.as_view(), name='update'),
    url(WORKBOOKS % 'detail', views.DetailView.as_view(), name='detail'),
]



# Copyright 2015 Huawei Technologies Co., Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse

from openstack_dashboard.test import helpers

from mistraldashboard import api
from mistraldashboard.test import helpers as test

INDEX_URL = reverse('horizon:mistral:tasks:index')


class TasksTest(test.TestCase):

    @helpers.create_mocks({api: ('task_list',)})
    def test_index(self):
        self.mock_task_list.return_value =\
            self.mistralclient_tasks.list()
        res = self.client.get(INDEX_URL)

        self.assertTemplateUsed(res, 'mistral/tasks/index.html')
        self.assertItemsEqual(res.context['table'].data,
                              self.mistralclient_tasks.list())
        self.mock_task_list.assert_called_once_with(helpers.IsHttpRequest())

    @helpers.create_mocks({api: ('task_get',)})
    def test_detail(self):
        task = self.mistralclient_tasks.list()[0]
        self.mock_task_get.return_value = task
        url = reverse('horizon:mistral:tasks:detail',
                      args=[task.id])
        res = self.client.get(url)
        self.assertTemplateUsed(res, 'mistral/tasks/detail.html')
        self.mock_task_get.assert_called_once_with(
            helpers.IsHttpRequest(), task.id)



# Copyright 2016 - Nokia.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json

from django.utils.translation import ugettext_lazy as _

from horizon import forms
from horizon import messages

from mistraldashboard import api
from mistraldashboard.default.utils import convert_empty_string_to_none
from mistraldashboard.handle_errors import handle_errors


class CreateForm(forms.SelfHandlingForm):
    name = forms.CharField(
        max_length=255,
        label=_("Name"),
        help_text=_('Cron Trigger name.'),
        required=True
    )

    workflow_id = forms.ChoiceField(
        label=_('Workflow ID'),
        help_text=_('Select Workflow ID.'),
        widget=forms.Select(
            attrs={'class': 'switchable',
                   'data-slug': 'workflow_select'}
        )
    )

    input_source = forms.ChoiceField(
        label=_('Input'),
        help_text=_('JSON of input values defined in the workflow. '
                    'Select either file or raw content.'),
        choices=[('file', _('File')),
                 ('raw', _('Direct Input'))],
        widget=forms.Select(
            attrs={'class': 'switchable',
                   'data-slug': 'inputsource'}
        )
    )
    input_upload = forms.FileField(
        label=_('Input File'),
        help_text=_('A local input to upload.'),
        widget=forms.FileInput(
            attrs={'class': 'switched',
                   'data-switch-on': 'inputsource',
                   'data-required-when-shown': 'true',
                   'data-inputsource-file': _('Input File')}
        ),
        required=False
    )
    input_data = forms.CharField(
        label=_('Input Data'),
        help_text=_('The raw contents of the input.'),
        widget=forms.widgets.Textarea(
            attrs={'class': 'switched',
                   'data-switch-on': 'inputsource',
                   'data-required-when-shown': 'true',
                   'data-inputsource-raw': _('Input Data'),
                   'rows': 4}
        ),
        required=False
    )

    params_source = forms.ChoiceField(
        label=_('Params'),
        help_text=_('JSON of params values defined in the workflow. '
                    'Select either file or raw content.'),
        choices=[('file', _('File')),
                 ('raw', _('Direct Input'))],
        widget=forms.Select(
            attrs={'class': 'switchable',
                   'data-slug': 'paramssource'}
        )
    )
    params_upload = forms.FileField(
        label=_('Params File'),
        help_text=_('A local input to upload.'),
        widget=forms.FileInput(
            attrs={'class': 'switched',
                   'data-switch-on': 'paramssource',
                   'data-required-when-shown': 'true',
                   'data-paramssource-file': _('Params File')}
        ),
        required=False
    )
    params_data = forms.CharField(
        label=_('Params Data'),
        help_text=_('The raw contents of the params.'),
        widget=forms.widgets.Textarea(
            attrs={'class': 'switched',
                   'data-switch-on': 'paramssource',
                   'data-required-when-shown': 'true',
                   'data-paramssource-raw': _('Params Data'),
                   'rows': 4}
        ),
        required=False
    )
    first_time = forms.CharField(
        label=_('First Time (YYYY-MM-DD HH:MM)'),
        help_text=_('Date and time of the first execution.'),
        widget=forms.widgets.TextInput(),
        required=False
    )
    schedule_count = forms.CharField(
        label=_('Count'),
        help_text=_('Number of desired executions.'),
        widget=forms.widgets.TextInput(),
        required=False
    )
    schedule_pattern = forms.CharField(
        label=_('Pattern (* * * * *)'),
        help_text=_('Cron Trigger pattern, mind the space between each char.'),
        widget=forms.widgets.TextInput(),
        required=False
    )

    def __init__(self, request, *args, **kwargs):
        super(CreateForm, self).__init__(request, *args, **kwargs)
        workflow_list = api.workflow_list(request)
        workflow_id_list = []
        for wf in workflow_list:
            workflow_id_list.append(
                (wf.id, "{id} ({name})".format(id=wf.id, name=wf.name))
            )

        self.fields['workflow_id'].choices = workflow_id_list

    def clean(self):
        cleaned_data = super(CreateForm, self).clean()
        cleaned_data['input'] = ""
        cleaned_data['params'] = ""

        if cleaned_data.get('input_upload'):
            files = self.request.FILES
            cleaned_data['input'] = files['input_upload'].read()
        elif cleaned_data.get('input_data'):
            cleaned_data['input'] = cleaned_data['input_data']

        del(cleaned_data['input_upload'])
        del(cleaned_data['input_data'])

        if len(cleaned_data['input']) > 0:
            try:
                cleaned_data['input'] = json.loads(cleaned_data['input'])
            except Exception as e:
                msg = _('Input is invalid JSON: %s') % str(e)
                raise forms.ValidationError(msg)

        if cleaned_data.get('params_upload'):
            files = self.request.FILES
            cleaned_data['params'] = files['params_upload'].read()
        elif cleaned_data.get('params_data'):
            cleaned_data['params'] = cleaned_data['params_data']

        del(cleaned_data['params_upload'])
        del(cleaned_data['params_data'])

        if len(cleaned_data['params']) > 0:
            try:
                cleaned_data['params'] = json.loads(cleaned_data['params'])
            except Exception as e:
                msg = _('Params is invalid JSON: %s') % str(e)
                raise forms.ValidationError(msg)

        return cleaned_data

    @handle_errors(_("Unable to create Cron Trigger"), [])
    def handle(self, request, data):
        data['input'] = convert_empty_string_to_none(data['input'])
        data['params'] = convert_empty_string_to_none(data['params'])
        data['schedule_pattern'] = convert_empty_string_to_none(
            data['schedule_pattern']
        )
        data['first_time'] = convert_empty_string_to_none(data['first_time'])
        data['schedule_count'] = convert_empty_string_to_none(
            data['schedule_count']
        )

        api.cron_trigger_create(
            request,
            data['name'],
            data['workflow_id'],
            data['input'],
            data['params'],
            data['schedule_pattern'],
            data['first_time'],
            data['schedule_count'],
        )
        msg = _('Successfully created Cron Trigger.')
        messages.success(request, msg)

        return True



# Copyright 2014 - StackStorm, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.template.defaultfilters import title
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import tables

from mistraldashboard import api
from mistraldashboard.default import smart_cell
from mistraldashboard.default.utils import humantime
from mistraldashboard.default.utils import label

smart_cell.init()


class UpdateRow(tables.Row):
    ajax = True

    def get_data(self, request, id):
        try:
            instance = api.task_get(request, id)
        except Exception as e:
            msg = _("Unable to get task by ID %(id)s: %(e)s.") % {
                'id': id, 'e': str(e)
            }
            exceptions.handle(request, msg)

        return instance


class TypeColumn(tables.Column):
    def get_link_url(self, datum):
        obj_id = datum.id
        url = ""
        action_execution_url = "horizon:mistral:action_executions:task"
        workflow_execution_url = "horizon:mistral:executions:tasks"
        if datum.type == "ACTION":
            url = action_execution_url
        elif datum.type == "WORKFLOW":
            url = workflow_execution_url

        return reverse(url, args=[obj_id])


class TaskTable(tables.DataTable):

    def getHoverHelp(data):
        if hasattr(data, 'state_info') and data.state_info:

                return {'title': data.state_info}

    STATE_STATUS_CHOICES = (
        ("success", True),
        ("error", False),
        ("idle", None),
        ("running", None),
    )

    id = tables.Column(
        "id",
        verbose_name=_("ID"),
        link="horizon:mistral:tasks:detail"
    )
    name = tables.Column(
        "name",
        verbose_name=_("Name")
    )
    workflow_execution_id = tables.Column(
        "workflow_execution_id",
        verbose_name=_("Workflow Execution ID"),
        link="horizon:mistral:executions:detail_task_id"
    )
    type = TypeColumn(
        "type",
        verbose_name=_("Type"),
        filters=[title],
        link=True
    )
    result = tables.Column(
        "",
        verbose_name=_("Result"),
        empty_value=_("View"),
        link="horizon:mistral:tasks:result",
        link_classes=("ajax-modal",)
    )
    published = tables.Column(
        "",
        verbose_name=_("Published"),
        empty_value=_("View"),
        link="horizon:mistral:tasks:published",
        link_classes=("ajax-modal",)
    )
    created_at = tables.Column(
        "created_at",
        verbose_name=_("Created at"),
        filters=[humantime]
    )
    updated_at = tables.Column(
        "updated_at",
        verbose_name=_("Updated at"),
        filters=[humantime]
    )

    state = tables.Column(
        "state",
        status=True,
        status_choices=STATE_STATUS_CHOICES,
        verbose_name=_("State"),
        filters=[label],
        cell_attributes_getter=getHoverHelp
    )

    class Meta(object):
        name = "tasks"
        verbose_name = _("Tasks")
        table_actions = (tables.FilterAction,)
        status_columns = ["state"]
        row_class = UpdateRow



# Copyright 2015 Huawei Technologies Co., Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse

from openstack_dashboard.test import helpers

from mistraldashboard import api
from mistraldashboard.test import helpers as test

INDEX_URL = reverse('horizon:mistral:executions:index')


class ExecutionsTest(test.TestCase):

    @helpers.create_mocks({api: ('pagination_list',)})
    def test_index(self):
        self.mock_pagination_list.return_value =\
            [self.mistralclient_executions.list(), False, False]
        res = self.client.get(INDEX_URL)
        self.assertTemplateUsed(res, 'mistral/executions/index.html')
        self.assertItemsEqual(res.context['table'].data,
                              self.mistralclient_executions.list())
        self.mock_pagination_list.assert_called_once_with(
            entity="executions",
            request=helpers.IsHttpRequest(),
            marker=None,
            sort_dirs='desc',
            paginate=True)

    @helpers.create_mocks({api: ('execution_update',)})
    def test_update_post(self):
        execution = self.mistralclient_executions.first()
        self.mock_execution_update.return_value = execution
        form_data = {
            "execution_id": execution.id,
            "description": "description"}
        res = self.client.post(
            reverse('horizon:mistral:executions:update_description',
                    args=(execution.id,)),
            form_data)
        self.assertNoFormErrors(res)
        self.mock_execution_update.assert_called_once_with(
            helpers.IsHttpRequest(), execution.id,
            "description", "description")

    @helpers.create_mocks({api: ('execution_get', 'task_list')})
    def test_detail(self):
        execution = self.mistralclient_executions.list()[0]
        tasks = self.mistralclient_tasks.list()
        self.mock_execution_get.return_value = execution
        self.mock_task_list.return_value = tasks
        url = reverse('horizon:mistral:executions:detail',
                      args=[execution.id])
        res = self.client.get(url)

        self.assertTemplateUsed(res, 'mistral/executions/detail.html')
        self.mock_execution_get.assert_called_once_with(
            helpers.IsHttpRequest(), execution.id)
        self.mock_task_list.assert_called_once_with(
            helpers.IsHttpRequest(), execution.id)



# Copyright 2015 Huawei Technologies Co., Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse

from openstack_dashboard.test import helpers as horizon_test

from mistraldashboard import api
from mistraldashboard.test import helpers as test

INDEX_URL = reverse('horizon:mistral:actions:index')


class ActionsTest(test.TestCase):

    @horizon_test.create_mocks({api: ('pagination_list',)})
    def test_index(self):
        self.mock_pagination_list.return_value =\
            [self.mistralclient_actions.list(), False, False]
        res = self.client.get(INDEX_URL)
        self.assertTemplateUsed(res, 'mistral/actions/index.html')
        self.mock_pagination_list.assert_called_once_with(
            entity="actions", request=horizon_test.IsHttpRequest(),
            marker=None, sort_keys='name', sort_dirs='desc',
            paginate=True, reversed_order=True)

    @horizon_test.create_mocks({api: ('action_create',)})
    def test_create_post(self):
        action = self.mistralclient_actions.first()
        self.mock_action_create.return_value = action
        url = reverse("horizon:mistral:actions:create")
        form_data = {
            'definition_source': 'raw',
            'definition_data': action.definition
        }
        res = self.client.post(url, form_data)

        self.assertNoFormErrors(res)
        self.mock_action_create.assert_called_once_with(
            horizon_test.IsHttpRequest(),
            action.definition)

    @horizon_test.create_mocks({api: ('action_update',)})
    def test_update_post(self):
        action = self.mistralclient_actions.first()
        self.mock_action_update.return_value = action
        url = reverse("horizon:mistral:actions:update")
        form_data = {
            'definition_source': 'raw',
            'definition_data': action.definition
        }
        res = self.client.post(url, form_data)

        self.assertNoFormErrors(res)
        self.mock_action_update.assert_called_once_with(
            horizon_test.IsHttpRequest(),
            action.definition)

    @horizon_test.create_mocks({api: ('action_get',)})
    def test_detail(self):
        action = self.mistralclient_actions.list()[0]
        self.mock_action_get.return_value = action
        url = reverse('horizon:mistral:actions:detail',
                      args=[action.id])
        res = self.client.get(url)

        self.assertTemplateUsed(res, 'mistral/actions/detail.html')
        self.mock_action_get.assert_called_once_with(
            horizon_test.IsHttpRequest(), action.id)



# Copyright 2016 - Nokia.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import reverse

from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy

from horizon import tables

from mistraldashboard import api
from mistraldashboard.default import smart_cell
from mistraldashboard.default import utils

smart_cell.init()


class UpdateRow(tables.Row):
    ajax = True

    def get_data(self, request, id):
        instance = api.action_execution_get(request, id)
        return instance


class DeleteActionExecution(tables.DeleteAction):
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Action Execution",
            u"Delete Action Executions",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Action Execution",
            u"Deleted Action Executions",
            count
        )

    def delete(self, request, action_execution_id):
        api.action_execution_delete(request, action_execution_id)


class UpdateActionExecution(tables.LinkAction):
    name = "updateAE"
    verbose_name = _("Update")
    url = "horizon:mistral:action_executions:update"
    classes = ("ajax-modal",)


class TaskExecutionIDColumn(tables.Column):
    def get_link_url(self, datum):
        task_url = "horizon:mistral:tasks:detail"
        obj_id = datum.task_execution_id
        return reverse(task_url, args=[obj_id])


class WorkflowNameColumn(tables.Column):
    def get_link_url(self, datum):
        workflow_url = "horizon:mistral:workflows:detail"
        obj_id = datum.workflow_name
        return reverse(workflow_url, args=[obj_id])


class ActionExecutionsTable(tables.DataTable):

    def getHoverHelp(data):
        if hasattr(data, 'state_info') and data.state_info:

                return {'title': data.state_info}

    STATE_STATUS_CHOICES = (
        ("success", True),
        ("error", False),
        ("idle", None),
        ("running", None),
        ("canceled", None),
    )

    id = tables.Column(
        "id",
        verbose_name=_("ID"),
        link="horizon:mistral:action_executions:detail"
    )
    name = tables.Column(
        "name",
        verbose_name=_("Name")
    )
    tags = tables.Column(
        "tags",
        verbose_name=_("Tags")
    )
    workflow_name = WorkflowNameColumn(
        "workflow_name",
        verbose_name=_("Workflow Name"),
        link=True
    )
    task_execution_id = TaskExecutionIDColumn(
        "task_execution_id",
        verbose_name=_("Task Execution ID"),
        link=True
    )
    task_name = tables.Column(
        "task_name",
        verbose_name=_("Task name")
    )
    description = tables.Column(
        "description",
        verbose_name=_("Description")
    )
    created_at = tables.Column(
        "created_at",
        verbose_name=_("Created at"),
        filters=[utils.humantime]
    )
    updated_at = tables.Column(
        "updated_at",
        verbose_name=_("Updated at"),
        filters=[utils.humantime]
    )
    accepted = tables.Column(
        "accepted",
        verbose_name=_("Accepted"),
        filters=[utils.booleanfield],
    )
    state = tables.Column(
        "state",
        status=True,
        status_choices=STATE_STATUS_CHOICES,
        verbose_name=_("State"),
        filters=[utils.label],
        cell_attributes_getter=getHoverHelp
    )

    class Meta(object):
        name = "actionExecutions"
        verbose_name = _("Action Executions")
        status_columns = ["state"]
        row_class = UpdateRow
        table_actions = (
            tables.FilterAction,
            DeleteActionExecution
        )
        row_actions = (UpdateActionExecution, DeleteActionExecution)



# Copyright 2015 Huawei Technologies Co., Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse

from openstack_dashboard.test import helpers

from mistraldashboard import api
from mistraldashboard.test import helpers as test

INDEX_URL = reverse('horizon:mistral:action_executions:index')


class ActionExecutionsTest(test.TestCase):

    @helpers.create_mocks({api: ('action_executions_list',)})
    def test_index(self):
        self.mock_action_executions_list.return_value =\
            self.mistralclient_action_executions.list()
        res = self.client.get(INDEX_URL)

        self.assertTemplateUsed(res, 'mistral/action_executions/index.html')
        self.mock_action_executions_list.assert_called_once_with(
            helpers.IsHttpRequest())

    @helpers.create_mocks({api: ('action_execution_update',)})
    def test_update_post(self):
        action_execution = self.mistralclient_action_executions.first()
        self.mock_action_execution_update.return_value = action_execution
        form_data = {"action_execution_id": action_execution.id,
                     "state": action_execution.state,
                     "output_source": "raw",
                     "output_data": action_execution.output}
        res = self.client.post(
            reverse('horizon:mistral:action_executions:update',
                    args=(action_execution.id,)),
            form_data)
        self.assertNoFormErrors(res)
        self.mock_action_execution_update.assert_called_once_with(
            helpers.IsHttpRequest(), action_execution.id,
            action_execution.state, action_execution.output)

    @helpers.create_mocks({api: ('action_execution_get',)})
    def test_detail(self):
        action_execution = self.mistralclient_action_executions.list()[0]
        self.mock_action_execution_get.return_value = action_execution
        url = reverse('horizon:mistral:action_executions:detail',
                      args=[action_execution.id])
        res = self.client.get(url)

        self.assertTemplateUsed(res, 'mistral/action_executions/detail.html')
        self.mock_action_execution_get.assert_called_once_with(
            helpers.IsHttpRequest(), action_execution.id)



# Copyright 2015 Huawei Technologies Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json

from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages

from mistraldashboard import api


class RunForm(forms.SelfHandlingForm):
    action_name = forms.CharField(
        label=_("Action"),
        required=True,
        widget=forms.TextInput(attrs={'readonly': 'readonly'})
    )
    input = forms.CharField(
        label=_("Input"),
        required=False,
        initial="{}",
        widget=forms.widgets.Textarea()
    )
    save_result = forms.CharField(
        label=_("Save result to DB"),
        required=False,
        widget=forms.CheckboxInput()
    )

    def handle(self, request, data):
        try:
            input = json.loads(data['input'])
        except Exception as e:
            msg = _('Action input is invalid JSON: %s') % str(e)
            messages.error(request, msg)

            return False

        try:
            params = {"save_result": data['save_result'] == 'True'}
            action = api.action_run(
                request,
                data['action_name'],
                input,
                params
            )
            msg = _('Run action has been created with name '
                    '"%s".') % action.name
            messages.success(request, msg)

            return True

        except Exception as e:
            # In case of a failure, keep the dialog open and show the error
            msg = _('Failed to run action "%(action_name)s"'
                    ' %(e)s:') % {'action_name': data['action_name'],
                                  'e': str(e)
                                  }
            messages.error(request, msg)

            return False


class CreateForm(forms.SelfHandlingForm):
    definition_source = forms.ChoiceField(
        label=_('Definition Source'),
        choices=[('file', _('File')),
                 ('raw', _('Direct Input'))],
        widget=forms.Select(
            attrs={'class': 'switchable',
                   'data-slug': 'definitionsource'})
    )
    definition_upload = forms.FileField(
        label=_('Definition File'),
        help_text=_('A local definition to upload.'),
        widget=forms.FileInput(
            attrs={'class': 'switched',
                   'data-switch-on': 'definitionsource',
                   'data-required-when-shown': 'true',
                   'data-definitionsource-file': _('Definition File')}
        ),
        required=False
    )
    definition_data = forms.CharField(
        label=_('Definition Data'),
        help_text=_('The raw contents of the definition.'),
        widget=forms.widgets.Textarea(
            attrs={'class': 'switched',
                   'data-switch-on': 'definitionsource',
                   'data-required-when-shown': 'true',
                   'data-definitionsource-raw': _('Definition Data'),
                   'rows': 4}
        ),
        required=False
    )

    def clean(self):
        cleaned_data = super(CreateForm, self).clean()

        if cleaned_data.get('definition_upload'):
            files = self.request.FILES
            cleaned_data['definition'] = files['definition_upload'].read()
        elif cleaned_data.get('definition_data'):
            cleaned_data['definition'] = cleaned_data['definition_data']
        else:
            raise forms.ValidationError(
                _('You must specify the definition source.'))

        return cleaned_data

    def handle(self, request, data):
        try:
            api.action_create(request, data['definition'])
            msg = _('Successfully created action.')
            messages.success(request, msg)

            return True
        except Exception as e:
            msg = _('Failed to create action: %s') % e
            redirect = reverse('horizon:mistral:actions:index')
            exceptions.handle(request, msg, redirect=redirect)


class UpdateForm(forms.SelfHandlingForm):
    definition_source = forms.ChoiceField(
        label=_('Definition Source'),
        choices=[('file', _('File')),
                 ('raw', _('Direct Input'))],
        widget=forms.Select(
            attrs={'class': 'switchable',
                   'data-slug': 'definitionsource'})
    )
    definition_upload = forms.FileField(
        label=_('Definition File'),
        help_text=_('A local definition to upload.'),
        widget=forms.FileInput(
            attrs={'class': 'switched',
                   'data-switch-on': 'definitionsource',
                   'data-definitionsource-file': _('Definition File')}
        ),
        required=False
    )
    definition_data = forms.CharField(
        label=_('Definition Data'),
        help_text=_('The raw contents of the definition.'),
        widget=forms.widgets.Textarea(
            attrs={'class': 'switched',
                   'data-switch-on': 'definitionsource',
                   'data-definitionsource-raw': _('Definition Data'),
                   'rows': 4}
        ),
        required=False
    )

    def clean(self):
        cleaned_data = super(UpdateForm, self).clean()

        if cleaned_data.get('definition_upload'):
            files = self.request.FILES
            cleaned_data['definition'] = files['definition_upload'].read()
        elif cleaned_data.get('definition_data'):
            cleaned_data['definition'] = cleaned_data['definition_data']
        else:
            raise forms.ValidationError(
                _('You must specify the definition source.'))

        return cleaned_data

    def handle(self, request, data):
        try:
            api.action_update(request, data['definition'])
            msg = _('Successfully updated action.')
            messages.success(request, msg)

            return True
        except Exception as e:
            msg = _('Failed to update action: %s') % e
            redirect = reverse('horizon:mistral:actions:index')
            exceptions.handle(request, msg, redirect=redirect)



# Copyright 2016 - Nokia.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import reverse
from django.urls import reverse_lazy

from django.utils.translation import ugettext_lazy as _
from django.views import generic

from horizon import forms
from horizon import tables

from mistraldashboard.action_executions import forms as action_execution_forms
from mistraldashboard.action_executions import tables as mistral_tables
from mistraldashboard import api
from mistraldashboard.default import utils
from mistraldashboard import forms as mistral_forms


def get_single_action_execution_data(request, **kwargs):
    action_execution_id = kwargs['action_execution_id']
    action_execution = api.action_execution_get(
        request,
        action_execution_id
    )
    return action_execution


class OverviewView(generic.TemplateView):
    template_name = 'mistral/action_executions/detail.html'
    page_title = _("Action Execution Details")
    workflow_url = 'horizon:mistral:workflows:detail'
    task_execution_url = 'horizon:mistral:tasks:detail'

    def get_context_data(self, **kwargs):
        context = super(OverviewView, self).get_context_data(**kwargs)
        action_execution = get_single_action_execution_data(
            self.request,
            **kwargs
        )
        if action_execution.workflow_name:
            action_execution.workflow_url = reverse(
                self.workflow_url,
                args=[action_execution.workflow_name])
        if action_execution.task_execution_id:
            action_execution.task_execution_url = reverse(
                self.task_execution_url,
                args=[action_execution.task_execution_id]
            )
        if action_execution.input:
            action_execution.input = utils.prettyprint(action_execution.input)
        if action_execution.output:
            action_execution.output = utils.prettyprint(
                action_execution.output
            )
        if action_execution.state:
            action_execution.state = utils.label(action_execution.state)
        action_execution.accepted = utils.booleanfield(
            action_execution.accepted
        )

        breadcrumb = [(action_execution.id, reverse(
            'horizon:mistral:action_executions:detail',
            args=[action_execution.id]
        ))]

        context["custom_breadcrumb"] = breadcrumb
        context['action_execution'] = action_execution

        return context


class CodeView(forms.ModalFormView):
    template_name = 'mistral/default/code.html'
    modal_header = _("Code view")
    form_id = "code_view"
    form_class = mistral_forms.EmptyForm
    cancel_label = "OK"
    cancel_url = reverse_lazy("horizon:mistral:action_executions:index")
    page_title = _("Code view")

    def get_context_data(self, **kwargs):
        context = super(CodeView, self).get_context_data(**kwargs)
        column = self.kwargs['column']
        action_execution = get_single_action_execution_data(
            self.request,
            **self.kwargs
        )
        io = {}

        if column == 'input':
            io['name'] = _('Input')
            io['value'] = utils.prettyprint(action_execution.input)
        elif column == 'output':
            io['name'] = _('Output')
            io['value'] = (
                utils.prettyprint(action_execution.output)
                if action_execution.output
                else _("No available output yet")
            )

        context['io'] = io

        return context


class IndexView(tables.DataTableView):
    table_class = mistral_tables.ActionExecutionsTable
    template_name = 'mistral/action_executions/index.html'

    def get_data(self):

        return api.action_executions_list(self.request)


class UpdateView(forms.ModalFormView):
    template_name = 'mistral/action_executions/update.html'
    modal_header = _("Update Action Execution")
    form_id = "update_action_execution"
    form_class = action_execution_forms.UpdateForm
    submit_label = _("Update")
    success_url = reverse_lazy("horizon:mistral:action_executions:index")
    submit_url = "horizon:mistral:action_executions:update"
    cancel_url = "horizon:mistral:action_executions:index"
    page_title = _("Update Action Execution")

    def get_initial(self):
        return {"action_execution_id": self.kwargs["action_execution_id"]}

    def get_context_data(self, **kwargs):
        context = super(UpdateView, self).get_context_data(**kwargs)
        context['submit_url'] = reverse(
            self.submit_url,
            args=[self.kwargs["action_execution_id"]]
        )

        return context


class FilteredByTaskView(tables.DataTableView):
    table_class = mistral_tables.ActionExecutionsTable
    template_name = 'mistral/action_executions/filtered.html'
    data = {}

    def get_data(self, **kwargs):
        task_id = self.kwargs['task_id']
        data = api.action_executions_list(self.request, task_id)

        return data



# Copyright 2015 Huawei Technologies Co., Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse

from openstack_dashboard.test import helpers

from mistraldashboard import api
from mistraldashboard.test import helpers as test

INDEX_URL = reverse('horizon:mistral:cron_triggers:index')


class CronTriggersTest(test.TestCase):

    @helpers.create_mocks({api: ('cron_trigger_list',)})
    def test_index(self):
        self.mock_cron_trigger_list.return_value =\
            self.mistralclient_cron_triggers.list()
        res = self.client.get(INDEX_URL)

        self.assertTemplateUsed(res, 'mistral/cron_triggers/index.html')
        self.mock_cron_trigger_list.assert_called_once_with(
            helpers.IsHttpRequest())

    @helpers.create_mocks({api: ('cron_trigger_create',
                                 'workflow_list')})
    def test_create_post(self):
        cron_trigger = self.mistralclient_cron_triggers.first()
        workflows = self.mistralclient_workflows.list()
        self.mock_cron_trigger_create.return_value = cron_trigger
        self.mock_workflow_list.return_value = workflows
        url = reverse("horizon:mistral:cron_triggers:create")
        form_data = {
            'name': cron_trigger.name,
            'workflow_id': '1',
            'input_source': 'raw',
            'input_data': '{"a":"b"}',
            'params_source': 'raw',
            'params_data': '{"a":"b"}',
            'schedule_pattern': cron_trigger.pattern,
            'first_time': cron_trigger.first_execution_time,
            'schedule_count': '1'
        }
        res = self.client.post(url, form_data)

        self.assertNoFormErrors(res)
        self.mock_cron_trigger_create.assert_called_once_with(
            helpers.IsHttpRequest(),
            cron_trigger.name, form_data["workflow_id"],
            {u'a': u'b'}, {u'a': u'b'},
            None, None,
            form_data["schedule_count"]
            )
        self.mock_workflow_list.assert_called_once_with(
            helpers.IsHttpRequest())

    @helpers.create_mocks({api: ('cron_trigger_get',)})
    def test_detail(self):
        cron_trigger = self.mistralclient_cron_triggers.list()[0]
        self.mock_cron_trigger_get.return_value = cron_trigger
        url = reverse('horizon:mistral:cron_triggers:detail',
                      args=[cron_trigger.id])
        res = self.client.get(url)

        self.assertTemplateUsed(res, 'mistral/cron_triggers/detail.html')
        self.mock_cron_trigger_get.assert_called_once_with(
            helpers.IsHttpRequest(), cron_trigger.id)



# Copyright 2014 - StackStorm, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _
from django.views import generic

from horizon import exceptions
from horizon import forms
from horizon import tables

from mistraldashboard import api
from mistraldashboard.workbooks import forms as mistral_forms
from mistraldashboard.workbooks import tables as mistral_tables


class IndexView(tables.DataTableView):
    table_class = mistral_tables.WorkbooksTable
    template_name = 'mistral/workbooks/index.html'

    def get_data(self):
        return api.workbook_list(self.request)


class DetailView(generic.TemplateView):
    template_name = 'mistral/workbooks/detail.html'
    page_title = _("Workbook Definition")

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        workbook = self.get_data(self.request, **kwargs)
        breadcrumb = [(workbook.name, reverse(
            'horizon:mistral:workbooks:detail',
            args=[workbook.name]
        ))]

        context["custom_breadcrumb"] = breadcrumb
        context['definition'] = workbook.definition

        return context

    def get_data(self, request, **kwargs):
        try:
            workbook_name = kwargs['workbook_name']
            workbook = api.workbook_get(request, workbook_name)
        except Exception:
            msg = _('Unable to get workbook "%s".') % workbook_name
            redirect = reverse('horizon:mistral:workbooks:index')
            exceptions.handle(self.request, msg, redirect=redirect)

        return workbook


class SelectDefinitionView(forms.ModalFormView):
    template_name = 'mistral/workbooks/select_definition.html'
    modal_header = _("Create Workbook")
    form_id = "select_definition"
    form_class = mistral_forms.DefinitionForm
    submit_label = _("Validate")
    submit_url = reverse_lazy("horizon:mistral:workbooks:select_definition")
    success_url = reverse_lazy('horizon:mistral:workbooks:create')
    page_title = _("Select Definition")

    def get_form_kwargs(self):
        kwargs = super(SelectDefinitionView, self).get_form_kwargs()
        kwargs['next_view'] = CreateView

        return kwargs


class ChangeDefinitionView(SelectDefinitionView):
    modal_header = _("Update Workbook")
    submit_url = reverse_lazy("horizon:mistral:workbooks:change_definition")
    success_url = reverse_lazy('horizon:mistral:workbooks:update')
    page_title = _("Update Definition")

    def get_form_kwargs(self):
        kwargs = super(ChangeDefinitionView, self).get_form_kwargs()
        kwargs['next_view'] = UpdateView

        return kwargs


class CreateView(forms.ModalFormView):
    template_name = 'mistral/workbooks/create.html'
    modal_header = _("Create Workbook")
    form_id = "create_workbook"
    form_class = mistral_forms.CreateForm
    submit_label = _("Create")
    submit_url = reverse_lazy("horizon:mistral:workbooks:create")
    success_url = reverse_lazy('horizon:mistral:workbooks:index')
    page_title = _("Create Workbook")

    def get_initial(self):
        initial = {}

        if 'definition' in self.kwargs:
            initial['definition'] = self.kwargs['definition']

        return initial


class UpdateView(CreateView):
    template_name = 'mistral/workbooks/update.html'
    modal_header = _("Update Workbook")
    form_id = "update_workbook"
    form_class = mistral_forms.UpdateForm
    submit_label = _("Update")
    submit_url = reverse_lazy("horizon:mistral:workbooks:update")
    page_title = _("Update Workbook")



# Copyright 2014 - StackStorm, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.views import generic

from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import tables

from mistraldashboard import api
from mistraldashboard.default import utils
from mistraldashboard.executions import forms as m_forms
from mistraldashboard.executions import tables as mistral_tables
from mistraldashboard import forms as mistral_forms


def get_single_data(request, id, type="execution"):
    """Get Execution or Task data by ID.

    :param request: Request data
    :param id: Entity ID
    :param type: Request dispatch flag, Default: Execution
    """

    if type == "execution":
        try:
            execution = api.execution_get(request, id)
        except Exception:
            msg = _('Unable to get execution by its ID"%s".') % id
            redirect = reverse('horizon:mistral:executions:index')
            exceptions.handle(request, msg, redirect=redirect)

        return execution

    elif type == "task":
        try:
            task = api.task_get(request, id)
        except Exception:
            msg = _('Unable to get task by its ID "%s".') % id
            redirect = reverse('horizon:mistral:tasks:index')
            exceptions.handle(request, msg, redirect=redirect)

        return task

    elif type == "task_by_execution":
        try:
            task = api.task_list(request, id)[0]
        except Exception:
            msg = _('Unable to get task by Execution ID "%s".') % id
            redirect = reverse('horizon:mistral:executions:index')
            exceptions.handle(request, msg, redirect=redirect)

        return task


class IndexView(tables.DataTableView):
    table_class = mistral_tables.ExecutionsTable
    template_name = 'mistral/executions/index.html'

    def has_prev_data(self, table):
        return self._prev

    def has_more_data(self, table):
        return self._more

    def get_data(self):
        executions = []
        prev_marker = self.request.GET.get(
            mistral_tables.ExecutionsTable._meta.prev_pagination_param,
            None
        )

        if prev_marker is not None:
            sort_dir = 'asc'
            marker = prev_marker
        else:
            sort_dir = 'desc'
            marker = self.request.GET.get(
                mistral_tables.ExecutionsTable._meta.pagination_param,
                None
            )

        try:
            executions, self._more, self._prev = api.pagination_list(
                entity="executions",
                request=self.request,
                marker=marker,
                sort_dirs=sort_dir,
                paginate=True
            )

            if prev_marker is not None:
                executions = sorted(
                    executions,
                    key=lambda execution: getattr(
                        execution, 'created_at'
                    ),
                    reverse=True
                )

        except Exception as e:
            self._prev = False
            self._more = False
            msg = _('Unable to retrieve executions list: %s') % e
            exceptions.handle(self.request, msg)

        return executions


class TasksView(tables.DataTableView):
    table_class = mistral_tables.ExecutionsTable
    template_name = 'mistral/executions/index_filtered_task.html'

    def has_prev_data(self, table):
        return self._prev

    def has_more_data(self, table):
        return self._more

    def get_data(self):
        executions = []
        prev_marker = self.request.GET.get(
            mistral_tables.ExecutionsTable._meta.prev_pagination_param,
            None
        )

        if prev_marker is not None:
            sort_dir = 'asc'
            marker = prev_marker
        else:
            sort_dir = 'desc'
            marker = self.request.GET.get(
                mistral_tables.ExecutionsTable._meta.pagination_param,
                None
            )

        try:
            executions, self._more, self._prev = api.pagination_list(
                entity="executions",
                request=self.request,
                marker=marker,
                sort_dirs=sort_dir,
                paginate=True,
                selector=self.kwargs['task_execution_id']
            )

            if prev_marker is not None:
                executions = sorted(
                    executions,
                    key=lambda execution: getattr(
                        execution, 'created_at'
                    ),
                    reverse=True
                )

        except Exception as e:
            self._prev = False
            self._more = False
            msg = (
                _('Unable to retrieve executions list of '
                  'the requested task: %s') % e
            )
            exceptions.handle(self.request, msg)

        return executions


class DetailView(generic.TemplateView):
    template_name = 'mistral/executions/detail.html'
    page_title = _("Execution Overview")
    workflow_url = 'horizon:mistral:workflows:detail'
    task_url = 'horizon:mistral:tasks:execution'

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        task = {}
        execution = {}
        if 'caller' in kwargs:
            if kwargs['caller'] == 'task':
                kwargs['task_id'] = kwargs['execution_id']
                del kwargs['execution_id']
                task = get_single_data(
                    self.request,
                    kwargs['task_id'],
                    "task"
                )
                execution = get_single_data(
                    self.request,
                    task.workflow_execution_id,
                )
        else:
            execution = get_single_data(
                self.request,
                kwargs['execution_id'],
            )
            task = get_single_data(
                self.request,
                self.kwargs['execution_id'],
                "task_by_execution"
            )

        execution.workflow_url = reverse(self.workflow_url,
                                         args=[execution.workflow_name])
        execution.input = utils.prettyprint(execution.input)
        execution.output = utils.prettyprint(execution.output)
        execution.params = utils.prettyprint(execution.params)
        execution.state = utils.label(execution.state)
        task.url = reverse(self.task_url, args=[execution.id])

        breadcrumb = [(execution.id, reverse(
            'horizon:mistral:executions:detail',
            args=[execution.id]
        ))]

        context["custom_breadcrumb"] = breadcrumb
        context['execution'] = execution
        context['task'] = task

        return context


class CodeView(forms.ModalFormView):
    template_name = 'mistral/default/code.html'
    modal_header = _("Code view")
    form_id = "code_view"
    form_class = mistral_forms.EmptyForm
    cancel_label = "OK"
    cancel_url = reverse_lazy("horizon:mistral:executions:index")
    page_title = _("Code view")

    def get_context_data(self, **kwargs):
        context = super(CodeView, self).get_context_data(**kwargs)
        execution = get_single_data(
            self.request,
            self.kwargs['execution_id'],
        )
        column = self.kwargs['column']
        io = {}
        if column == 'input':
            io['name'] = _('Input')
            io['value'] = execution.input = utils.prettyprint(execution.input)
        elif column == 'output':
            io['name'] = _('Output')
            io['value'] = execution.output = utils.prettyprint(
                execution.output
            )

        context['io'] = io

        return context


class UpdateDescriptionView(forms.ModalFormView):
    template_name = 'mistral/executions/update_description.html'
    modal_header = _("Update Execution Description")
    form_id = "update_execution_description"
    form_class = m_forms.UpdateDescriptionForm
    submit_label = _("Update")
    success_url = reverse_lazy("horizon:mistral:executions:index")
    submit_url = "horizon:mistral:executions:update_description"
    cancel_url = "horizon:mistral:executions:index"
    page_title = _("Update Execution Description")

    def get_initial(self):
        return {"execution_id": self.kwargs["execution_id"]}

    def get_context_data(self, **kwargs):
        context = super(UpdateDescriptionView, self).get_context_data(**kwargs)
        context['submit_url'] = reverse(
            self.submit_url,
            args=[self.kwargs["execution_id"]]
        )

        return context
